#ifndef TILE_WIDTH
#define TILE_WIDTH 32
#endif

#ifndef TILE_HEIGHT
#define TILE_HEIGHT 32
#endif

#define mouse.buttons 1
#define MOUSE_RIGHT 2
#define MOUSE_MIDDLE 4

screen_loc
	var
		text
		map_id = ":map"
		tile_x
		step_x
		tile_y
		step_y

mouse
	var
		x = 0
		y = 0
		screen_loc = ""
		map = ":map"
		mouse.buttons = 0
		offset_x = 0
		offset_y = 0

client
	var tmp/mouse/mouse = new

	New()
		. = ..()
		screen += new /atom/movable {
			name = ""
			layer = -1#INF
			mouse_opacity = 2
			screen_loc = "SOUTHWEST to NORTHEAST"
		}

	proc
		// Parses a string in the form
		// 	"[map_id]:[tile_x]:[step_x],[tile_y]:[step_y]"
		// or
		// 	"[tile_x]:[step_x],[tile_y]:[step_y]"
		// Assumes map_id doesn't start with a number.
		// Returns a /screen_loc object.
		ParseScreenLoc(ScreenLoc)
			// string to be modified in parsing
			var s = ScreenLoc

			// return values
			var screen_loc/screen_loc = new
			screen_loc.text = ScreenLoc

			// map ID
			if(!text2num(s))
				var colon = findtext(s, ":")
				screen_loc.map_id = copytext(s, 1, colon)
				s = copytext(s, colon+1)

			// first number
			screen_loc.tile_x = text2num(s)
			s = copytext(s, length("[screen_loc.tile_x]")+2)

			// second number
			screen_loc.step_x = text2num(s)
			s = copytext(s, length("[screen_loc.step_x]")+2)

			// third number
			screen_loc.tile_y = text2num(s)
			s = copytext(s, length("[screen_loc.tile_y]")+2)

			// fourth number
			screen_loc.step_y = text2num(s)

			if(length(s) != length("[screen_loc.step_y]"))
				throw EXCEPTION("Parsing failed (did map ID start with a number?)")

			return screen_loc

		SetMouseScreenLoc(ScreenLoc) if(ScreenLoc && mouse.screen_loc != ScreenLoc)
			var screen_loc/screen_loc = ParseScreenLoc(ScreenLoc)
			mouse.screen_loc = screen_loc.text
			mouse.map = screen_loc.map_id
			mouse.x = screen_loc.step_x + (screen_loc.tile_x - 1) * TILE_WIDTH - mouse.offset_x
			mouse.y = screen_loc.step_y + (screen_loc.tile_y - 1) * TILE_HEIGHT - mouse.offset_y
			MouseMoved()

		MouseMoved()

	MouseEntered(object, location, control, params)
		SetMouseScreenLoc(params2list(params)["screen-loc"])
		..()

	MouseExited(object, location, control, params)
		SetMouseScreenLoc(params2list(params)["screen-loc"])
		..()

	MouseMove(object, location, control, params)
		SetMouseScreenLoc(params2list(params)["screen-loc"])
		..()

	MouseDrag(
	src_object, over_object,
	src_location, over_location,
	src_control, over_control, params)
		SetMouseScreenLoc(params2list(params)["screen-loc"])
		..()

	MouseDrop(
	src_object, over_object,
	src_location, over_location,
	src_control, over_control, params)
		var p[] = params2list(params)
		if(p["left"]) mouse.buttons |= mouse.buttons
		else if(p["right"]) mouse.buttons |= MOUSE_RIGHT
		else if(p["middle"]) mouse.buttons |= MOUSE_MIDDLE
		SetMouseScreenLoc(p["screen-loc"])
		..()

	MouseUp(object, location, control, params)
		var p[] = params2list(params)
		if(p["left"]) mouse.buttons &= ~mouse.buttons
		else if(p["right"]) mouse.buttons &= ~MOUSE_RIGHT
		else if(p["middle"]) mouse.buttons &= ~MOUSE_MIDDLE
		SetMouseScreenLoc(p["screen-loc"])
		..()

	MouseDown(object, location, control, params)
		var p[] = params2list(params)
		if(p["left"]) mouse.buttons |= mouse.buttons
		else if(p["right"]) mouse.buttons |= MOUSE_RIGHT
		else if(p["middle"]) mouse.buttons |= MOUSE_MIDDLE
		SetMouseScreenLoc(p["screen-loc"])
		..()
